package com.nisum.ecomservice.service;

import com.nisum.ecomservice.dto.*;
import com.nisum.ecomservice.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class OrderService {

    @Autowired
    ProductService productService;

    @Value("${orders.apiUrl}")
    private String orderManagementUrl;

    public Mono<Order> postOrder(OrderRequest orderRequest) {

        //create user object from user object details
        User user = orderRequest.getUser();
        Address address = orderRequest.getAddress();
        CartPostDTO cart = orderRequest.getCart();
        //for each item grab product details from products API
        List<Mono<Product>> productsToOrder = cart.getCartItems().stream()
                .map(cartItem -> cartItem.getProductRef().getSku())
                .map(sku -> productService.getProductBySku(sku))

                .collect(Collectors.toList());

        Flux<Product> productFlux = Flux.concat(productsToOrder);

        Flux<OrderItem> orderItems = productFlux.map(product -> {
            OrderItem orderItem = new OrderItem();
            orderItem.setItemId(product.getSku());
            orderItem.setItemName(product.getProductName());
            orderItem.setItemSku(product.getSku());
            orderItem.setItemPrice(product.getPrice());

            List<CartItemDTO> items = cart.getCartItems().stream().filter(cartItem -> cartItem.getProductRef().getSku()
                    .equals(product.getSku()))
                    .collect(Collectors.toList());

            Integer quantity = items.get(0).getQuantity();

            orderItem.setItemQuantity(quantity);

            return orderItem;
        }).flatMap(orderItem -> {
            Flux<Promotion> promotionFlux = productService
                    .getPromotionBySkus(Arrays.asList(orderItem.getItemSku()));

            Mono<OrderItem> orderItemMono = promotionFlux
                    .collectList()
                    .map(promotionList -> {
                        System.out.println("Give me promo list");
                        if (promotionList.size() > 0) {
                            Promotion promotion = promotionList.get(0);
                            if (orderItem.getItemQuantity().intValue() >= promotion.getMinimumQuantity().intValue()) {
                                Float originalPrice = orderItem.getItemPrice();
                                Float newPrice = originalPrice * ((100 - promotion.getDiscountPercentage()) / 100);
                                orderItem.setItemPrice(newPrice);
                            }
                        }
                        return orderItem;
                    });
            return orderItemMono;
        });
       //falttens the things

        Mono<Mono<Order>> map = orderItems.collectList().map(itemList -> {
            OrderSubmission orderSubmission = new OrderSubmission();
            orderSubmission.setOrderItems(itemList);
            orderSubmission.setCustomerAddress(address);
            orderSubmission.setCustomerId(user.getUserId());
            orderSubmission.setCustomerEmailAddress(user.getEmail());
            System.out.println(orderSubmission);
            return orderSubmission;
        }).map(orderSubmission -> postOrderToAPI(orderSubmission));


        //post order to OMS team
        //retrieve their response

        return map.flatMap(orderMono -> orderMono);

    }

    public Mono<Order> postOrderToAPI(OrderSubmission orderSubmission){
        return WebClient.create(String.format("%s/api/orders", orderManagementUrl))
                .post()
                .bodyValue(orderSubmission)
                .retrieve()
                .bodyToMono(Order.class);
    }

    public Flux<Order> getOrderFromOmsAPI(String userId){
        return WebClient.create(String.format("%s/api/orders/customer/%s", orderManagementUrl,userId))
                .get()
                .retrieve()
                .bodyToFlux(Order.class);
    }
}
/*
 .map(productMono -> {
                    productMono
                            .map(product -> {
                                OrderItem orderItem = new OrderItem();
                                orderItem.setItemId(product.getSku());
                                orderItem.setItemName(product.getProductName());
                                orderItem.setItemSku(product.getSku());
                                orderItem.setItemPrice(product.getPrice());

                                List<CartItem> items = user.getCart().stream().filter(cartItem -> cartItem.getProductRef().getSku()
                                        .equals(product.getSku()))
                                        .collect(Collectors.toList());
                                Integer quantity = items.get(0).getQuantity();

                                orderItem.setItemQuantity(quantity);
                                System.out.println(orderItem);
                                return  orderItem;
                            })
                })
 */