import React, { useState, useEffect } from 'react'

export default function ReviewOrder(props) {


    let calcTotalBeforeTax = (itemsTotal, shippingTotal) => {
        return parseFloat(itemsTotal) + parseFloat(shippingTotal)
    }

    let calcTaxAmount = (itemsTotal, shippingTotal, taxRate) => {
        return calcTotalBeforeTax(itemsTotal, shippingTotal) * taxRate
    }

    let calcGrandTotal = (itemsTotal, shippingTotal, taxRate) => {
        return calcTotalBeforeTax(itemsTotal, shippingTotal) * (parseFloat(1.00) + parseFloat(taxRate))
    }

    return (
        <div className="order-component" id="ReviewOrder">

            <div className="OrderDirectionsElement" id="OrderSummary">
                <p>Step 3 - Order Summary</p>
            </div>


            {
                props.loading === 0 ?

                    <div id="orderSummaryContainer">
                        <div className="OrderSummaryElement" id="itemsPrice">
                            <div>
                                Items ({props.numCartItems}):
                            </div>

                            <div>
                                ${parseFloat(props.itemsTotal).toFixed(2)}
                            </div>
                        </div>


                        <div className="OrderSummaryElement" id="promotionsApplied">
                            <div>
                                Promotions Discount:
                            </div>

                            <div>
                                <span style={{color: 'green', fontWeight: 'bold'}}>
                                    -${parseFloat(props.promotionsDiscount).toFixed(2)}
                                </span>
                            </div>
                        </div>

                        <div className="OrderSummaryElement" id="shippingHandling">
                            <div id="shippingHandlingLabel">
                                Shipping and Handling:
                            </div>

                            <div id="shippingHandlingPrice">
                                {/* ${parseFloat(props.shippingHandling).toFixed(2)} */}
                                <span style={{color: 'green', fontWeight: 'bold'}}>FREE</span>
                            </div>
                        </div>

                        <div className="OrderSummaryElement" id="beforeTax">
                            <div id="beforeTaxLabel">
                                Total Before Tax
                            </div>

                            <div id="beforeTaxPrice">
                                ${calcTotalBeforeTax(props.itemsTotal - props.promotionsDiscount, props.shippingHandling).toFixed(2)}
                            </div>
                        </div>

                        <div className="OrderSummaryElement" id="taxCalculated">
                            <div id="taxCalculatedLabel">
                                Estimated Tax:
                            </div>

                            <div id="taxCalculatedPrice">
                                ${calcTaxAmount(props.itemsTotal - props.promotionsDiscount, props.shippingHandling, props.taxRate).toFixed(2)}
                            </div>
                        </div>

                        <div className="OrderSummaryElement" id="orderTotal">
                            <div id="orderTotalLabel">
                                Order Total:
                            </div>

                            <div id="orderTotalPrice">
                                ${calcGrandTotal(props.itemsTotal - props.promotionsDiscount, props.shippingHandling, props.taxRate).toFixed(2)}
                            </div>
                        </div>
                    </div>
                    :

                    <div id="loadingElement">
                        <div class="lds-ring"><div></div><div></div><div></div><div></div></div>
                    </div>
            }

        </div>
    )
}
