import React, { useState, useEffect } from 'react'
import { useSelector, useDispatch } from 'react-redux'
import ShippingAddress from './ShippingAddress.js'
import PaymentMethod from './PaymentMethod.js'
import ReviewOrder from './ReviewOrder.js'
import SubmitOrder from './SubmitOrder.js'
import './checkout.css'

import {dispatchOrderInfo} from './../../actions/checkout_actions'
import { Redirect } from 'react-router'
import { updateUserCart } from './../../actions/cart_actions'
import { calcTotalPrice, calcNumItems, storeCartItemsInProcessing, calcTotalPromotions } from './../../actions/cart_processing_actions'
export default function Checkout() {
    const dispatch = useDispatch()
    ///////////////////////
    // Shipping Info State
    ///////////////////////
    // const [firstName, setFirstName] = useState("Guy")
    // const [lastName, setLastName] = useState("Fieri")
    // const [shippingAddress, setShippingAddress] = useState("1234 Flavortown Dr")
    // const [aptSuiteNo, setAptSuiteNo] = useState("Apt 317")
    // const [city, setCity] = useState("Santa Rosa")
    // const [state, setState] = useState("California")
    // const [zipCode, setZipCode] = useState("90210")
    const [firstName, setFirstName] = useState("")
    const [lastName, setLastName] = useState("")
    const [shippingAddress, setShippingAddress] = useState("")
    const [aptSuiteNo, setAptSuiteNo] = useState("")
    const [city, setCity] = useState("")
    const [state, setState] = useState("")
    const [zipCode, setZipCode] = useState("")
    ///////////////////////
    // Billing Info State
    ///////////////////////
    const [cardNumber, setCardNumber] = useState(["123456789123456"])
    const [cardHolderName, setCardHolderName] = useState(["Guy Fieri"])
    const [expirationDate, setExpirationDate] = useState(["05/2026"])
    const [cvv, setCVV] = useState(["123"])
    ///////////////////////
    // Order Review / Summary State
    ///////////////////////
    const numCartItems = useSelector(state => state.cartProcessing.numItems)
    const itemsTotal = useSelector(state => state.cartProcessing.totalPrice)
    const promotionsDiscount = useSelector(state => state.cartProcessing.totalPromotions)
    const taxRate = 0.0
    const shippingHandling = 0.00
    ///////////////////////
    // Submit Button State
    ///////////////////////
    const [submitButtonActive, setSubmitButtonActive] = useState(0)
    const [allFieldsValidated, setAllFieldsValidated] = useState(0)
    const [errorWhileValidating, setErrorWhileValidating] = useState(0)
    ///////////////////////
    // Form Submission State
    ///////////////////////
    const user = useSelector(state => state.user.currentUser)
    const cart = useSelector(state => state.cart)
    const [loading, setLoading] = useState(0)

    useEffect(() => {
        if (firstName && lastName && shippingAddress && city && state && zipCode) {
            console.log("All Fields Validated!")
            setAllFieldsValidated(1)
        }
    }, [firstName, lastName, shippingAddress, city, state, zipCode])
    
    useEffect(() => {
        if (submitButtonActive === 1) {
            if (allFieldsValidated) {
                setErrorWhileValidating(0)
                handleSubmit()
                console.log("Successful submission request")
                setSubmitButtonActive(0)
                const cartUpdateObj = {
                    userId: user.email,
                    cartItems: []
                }

                // clear user cart now that order has been placed
                setLoading(1)
                dispatch(updateUserCart(cartUpdateObj, cartUpdateObj.userId))
                dispatch(storeCartItemsInProcessing([]))
                dispatch(calcTotalPrice([]))
                dispatch(calcNumItems([]))
                dispatch(calcTotalPromotions([]))
            }
            else {
                setErrorWhileValidating(1)
                setSubmitButtonActive(0)
            }
        }
        else { ; }
    }, [submitButtonActive])
    
    let handleSubmit = () => {
        console.log("Submitting Order!")
        let orderInfo = {
            "user": {
                "userId": user.userId,
                "email": user.email,
                "firstName": user.firstName,
                "lastName": user.lastName,
                "accessToken": user.accessToken
            },
            "address": {
                "state": state,
                "city": city,
                "zip": zipCode,
                "street": shippingAddress
            },
            "cart": {
                "id": "something",
                "userId": user.userId,
                "cartItems": cart
            }
        }

        dispatch(dispatchOrderInfo(orderInfo))
    }

    //redirect to order confirmation page once reponse is recieved
    const {orderResponse} = useSelector(state => state.orderStatus);
    if (orderResponse.id) return <Redirect to="/order-confirmation" />
    
    return (
        <div id="checkout-container">
            {/* Collects User's shipping info */}
            <ShippingAddress
                captureFirstName={setFirstName}
                captureLastName={setLastName}
                captureShippingAddress={setShippingAddress}
                captureAptSuiteNo={setAptSuiteNo}
                captureCity={setCity}
                captureState={setState}
                captureZipCode={setZipCode}
            />
            {/* Collects User's payment info */}
            <PaymentMethod
                cardNumber={cardNumber}
                cardHolderName={cardHolderName}
                expirationDate={expirationDate}
                cvv={cvv}
                captureCardNumber={setCardNumber}
                captureCardHolderName={setCardHolderName}
                captureExpirationDate={setExpirationDate}
                captureCVV={setCVV}
            />
            {/* Displays info about order. Calculates price with tax / shipping applied */}
            <ReviewOrder
                numCartItems={numCartItems}
                promotionsDiscount={promotionsDiscount}
                itemsTotal={itemsTotal}
                shippingHandling={shippingHandling}
                taxRate={taxRate}
                loading={loading}
            />
            {/* Request to submit happens here, initiates input validation and sends if success */}
            <SubmitOrder
                submitButtonActive={submitButtonActive}
                setSubmitButtonActive={setSubmitButtonActive}
                errorWhileValidating={errorWhileValidating}
                allFieldsValidated={allFieldsValidated}
            />
        </div>
    )
}