import React, { useState, useEffect } from 'react'
import CartItem from './CartItem.js'
import './shopping-cart.css'
import { useSelector, useDispatch } from 'react-redux'
import { Link, Redirect } from 'react-router-dom'
import { updateUserCart} from './../../actions/cart_actions'
import { calcTotalPrice, calcNumItems, storeCartItemsInProcessing, calcTotalPromotions } from './../../actions/cart_processing_actions'
export default function ShoppingCart() {
  
  const dispatch = useDispatch()
  const userSession = useSelector(state => state.user.currentUser)
  const allProducts = useSelector(state => state.market.products)
  const allPromotions = useSelector(state => state.market.promotions)
  const [cartRefs, setCartRefs] = useState(useSelector(state => state.cart))
  const [cartItems, setCartItems] = useState([])
  ////////////////////////////////////////////////////////////////
  // Map Product Refs to Products that exist in redux global state
  ////////////////////////////////////////////////////////////////
  useEffect(() => {
    const productsFromRefs = []
    cartRefs.map((cartRef) => {
      productsFromRefs.push((
        {
          product: allProducts.filter((currProduct) => (currProduct.sku === cartRef.productRef.sku))[0],
          quantity: cartRef.quantity
        }))
    })
    console.log("NEW PRODUCTS FROM REFS YO")
    console.log(productsFromRefs)
    setCartItems(productsFromRefs)
  }, [cartRefs])
  ////////////////////////////////
  // Delete Item from Cart
  ////////////////////////////////
  let handleDelete = (skuToBeDeleted) => {
    const newCartRefs = cartRefs.filter((cartRef) => !(cartRef.productRef.sku === skuToBeDeleted))
    setCartRefs(newCartRefs)
  }
  ////////////////////////////////
  // Update Quantity of an item
  ////////////////////////////////
  let handleQuantityUpdate = (skuToBeUpdated, newQuantity) => {
    const newCartRefs = []
    cartRefs.map( (cartRef) => {
      if (cartRef.productRef.sku === skuToBeUpdated) {
        let temp = cartRef
        temp.quantity = newQuantity
        newCartRefs.push(temp)
      }
      else {
        newCartRefs.push(cartRef)
      }
    })
    setCartRefs(newCartRefs)
  }
  ////////////////////////////////////////////////////////////////
  // Sync Cart with backend and redux global state
  ////////////////////////////////////////////////////////////////
  useEffect(() => {
    if (!userSession) return;
    const cartUpdateObj = {
      userId: userSession.email,
      cartItems: cartRefs
    }
    dispatch(updateUserCart(cartUpdateObj, cartUpdateObj.userId))
    dispatch(storeCartItemsInProcessing(cartItems))
    dispatch(calcTotalPrice(cartItems))
    dispatch(calcNumItems(cartItems))
    dispatch(calcTotalPromotions(cartItems))
  }, [cartItems])

  return (
    <div id="shoppingCartContainer">
      {/* map each cart item into CartItem component */}
      <div id="cartItemList">
        {cartItems.map((currItem) => {
          return (
            <CartItem
              productInfo={currItem.product}
              quantity={currItem.quantity}
              handleDelete={handleDelete}
              handleQuantityUpdate={handleQuantityUpdate}
              key={currItem.product.sku}
            />
          )
        })}
      </div>

        {
          cartItems.length > 0 ? (
            <Link id="checkoutButtonContainer" to="/checkout">
              <button id="checkoutButton">Proceed To Checkout</button>
            </Link>
          ) : (
            <div>Add more items to your cart!</div>
          )
        }
      


    </div>
  )
}
